/*******************************************************************
 Copyright (C) 2004 Thomas Kunz, CRC Canada, BCAST for IPv4.
 DISTRIBUTED WITH NO WARRANTY, EXPRESS OR IMPLIED.
 See the GNU Library General Public License (file COPYING in
 directory application) for conditions of use and redistribution.
*********************************************************************/

/*
 * Constants defined for BCAST protocol
 */

#ifndef bcast_const_hh

/* each instance of BCAST is uniquely identified by an ID. We
 * choose to make this the IP address of the main interface.
 * which has name BCAST_INTERFACE.
 */

#define BCAST_INTERFACE	"eth0"

/* constants to propagate a BCAST CONTROL message to all neighbours, 
 * using multicasting over one hop (all nodes register with the 
 * well-known multicast group to exchange control information...)
 */

#define BCAST_GROUP		"224.0.0.128" // not currently assigned by IANA
#define BCAST_PORT		1500          // more or less arbitrary, > 1023
#define BCAST_TTL		1             // propate ONE hop
#define BCAST_MAX_MSG		2000          // hopefully never more

/* the main loop is event-driven, so periodically poll the various
 * file descriptors whether new data (control message or user data)
 * is available to be read in, preventing BLOCKING I/O.
 */

#define BCAST_POLL_PERIOD       20    // poll every 20 milliseconds

/* some other events get scheduled periodically: clean up list of
 * unique packet IDs, check for expired neighbors, etc. The periodicity
 * of these events is defined by the following constants.
 */

#define CLEANUP_PERIOD  5000         // every 5000 ms

// how much to delay packet retransmission (scaling factor)
#define BCAST_DELAY		100          // in ms

// how much time to delay NACK'd packet retransmissions (scaling factor)
#define PENDING_DELAY	10           // in ms

/* we use NACKs up to MAX_NACK times for a specific packet that is deemed 
 * lost. After trying for MAX_NACK times, we will give up. The timer is 
 * cancelled when the requested packet is received.
 */

#define MAX_NACK 3
#define NACK_INITIAL_DELAY 10	// upper bound (in ms) on initial NACK jitter 
#define NACK_TIMEOUT 1000		// timeout and try again after x ms

// constants to control frequency of NACKs (the throttling mechanism)
#define MIN_DIFF	700000	// do not send too many NACKs per unit time
#define PACKET_COUNT	30	// if we exceed this, do not issue NACK

/* We periodically send HELLO messages to neighbors. To avoid HELLO
 * message collisions, they are randomly scheduled to occur between
 * [0.75 * HELLO_INTERVAL, 1.25 * HELLO_INTERVAL]. If we miss more than
 * ALLOWED_HELLO_LOSS messages from the neighbors, we assume the neighor
 * is gone.
 */

#define DEFAULT_HELLO_INTERVAL  2             // in seconds
#define ALLOWED_HELLO_LOSS      1             // packets
#define MaxHelloScaling         1.25 
#define MinHelloScaling         0.75 

/* we also buffer user data in a local buffer, a buffer whose size
 * is determined by configuration parameters
 */

#define DEFAULT_BUFFER_SIZE     10            // at most last 10 packets

#endif

