/*******************************************************************
 Copyright (C) 2004 Thomas Kunz, CRC Canada, BCAST for IPv4.
 DISTRIBUTED WITH NO WARRANTY, EXPRESS OR IMPLIED.
 See the GNU Library General Public License (file COPYING in
 directory application) for conditions of use and redistribution.
*********************************************************************/

// Driver for BCAST protocol

#include "bcast_module.h"

#include "libxorp/xorp.h"
#include "libxorp/xlog.h"
#include "libxorp/eventloop.hh"
#include "libxorp/exceptions.hh"
#include "libxipc/xrl_std_router.hh"

#include <stdlib.h>

#include "bcast_tgt.hh"

static const char* g_tgt_name = "bcast";

static void
run_bcast()
{
     EventLoop e;

    // Configure target for bcast    
    XrlStdRouter tgt_router(e, g_tgt_name); 
    XrlBcastTarget tgt_target(&e, &tgt_router);
    
    bool timeout = false;
    XorpTimer t = e.set_flag_after_ms(2000, &timeout);
    while (timeout == false && tgt_router.ready() == false)
	e.run();

    if (timeout) {
	fprintf(stderr, "Timed out connecting to finder\n");
	exit(-1);
    }

    // run until shutdown externally
    while (true)
	e.run();
}

int main(int /* argc */, const char* argv[])
{
    xlog_init(argv[0], 0);
    xlog_set_verbose(XLOG_VERBOSE_HIGH);
    xlog_add_default_output();
    xlog_start();

#if 1
    //
    // Root test, can't run if not root.
    //
    if (geteuid() != 0) {
	fprintf(stderr, "%s\n*\n", string(79, '*').c_str());
	fprintf(stderr, "* This program needs root privileges.\n");
	fprintf(stderr, "* Returning success even though not run.\n");
	fprintf(stderr, "*\n%s\n", string(79, '*').c_str());
	return EXIT_SUCCESS;
    }
#endif

    try {
	run_bcast();
    } catch (...) {
	xorp_catch_standard_exceptions();
    }

    xlog_stop();
    xlog_exit();

    return 0;
}
