/*******************************************************************
 Copyright (C) 2004 Thomas Kunz, CRC Canada, BCAST for IPv4.
 DISTRIBUTED WITH NO WARRANTY, EXPRESS OR IMPLIED.
 See the GNU Library General Public License (file COPYING in
 directory application) for conditions of use and redistribution.
*********************************************************************/

/* BCAST buffers packets before they are retransmitted, delaying them
 * by a certain amount of time. The necessary functionality is implemented
 * in class BCAST_BUFFER.
 */

#ifndef _buffer_hh_

#define _buffer_hh_

#include "bcast.hh"
#include "bcast_const.hh"
#include <pcap.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <sys/types.h>

#include <list>

class XrlBcastTarget;

class bcast_buffer_entry {
public:
  bcast_buffer_entry(char *msg, int msg_len, in_addr_t addr, u_int16_t port,
                     struct bcastHeader msg_hdr, struct timeval tv, 
                     list<in_addr_t> uncovered_neighbors)
  { memcpy(buf, msg, msg_len);
    buf_len = msg_len;
    dst_addr = addr;
    dst_port = port;
    memcpy((char *)&bcast_hdr, (char *)&msg_hdr, sizeof(bcastHeader));
    memcpy((char *)&expiry, (char *)&tv, sizeof(struct timeval));
    neighbors = uncovered_neighbors;
  };

  bool compare_neighbor_list(struct bcastHeader msg_hdr, list<in_addr_t> dests);
  void adjust_timer(struct bcastHeader msg_hdr, struct timeval *tv);
  void send_msg(XrlBcastTarget *obj);
  struct timeval& time_val() { return expiry;};

private:
  char buf[BCAST_MAX_MSG];      // the intercepted message (payload only)
  int buf_len;                  // length of payload
  in_addr_t dst_addr;		// multicast group that this packet is being sent to
  u_int16_t dst_port;		// destination port number
  struct bcastHeader bcast_hdr; // protocol-specific BCAST header
  struct timeval expiry;        // when to transmit the message
  list<in_addr_t> neighbors;    // pending list of neighbors to be reached
};

// declare comparison parameter for entities of type BCAST_BUFFER_ENTRY
// bool cmp_entries(bcast_buffer_entry b1, bcast_buffer_entry b2);

class bcast_buffer {
public:
  bcast_buffer(XrlBcastTarget *obj){ bcast_obj = obj;};
  void add_to_buffer(char *msg, int msg_len, in_addr_t addr, u_int16_t port,
                     struct bcastHeader msg_hdr, double delay, list<in_addr_t> nb);
  void check_against_buffer(struct bcastHeader msg_hdr, list<in_addr_t> dests);
  void send_first_msg();
  struct timeval& expiry_first_msg();

private:
  void schedule_first_message(); // set timer for first pending message

  list<bcast_buffer_entry> buffer;
  XrlBcastTarget *bcast_obj;
};

void handle_buffer(bcast_buffer *buf);

#endif
