/*******************************************************************
 Copyright (C) 2004 Thomas Kunz, CRC Canada, BCAST for IPv4.
 DISTRIBUTED WITH NO WARRANTY, EXPRESS OR IMPLIED.
 See the GNU Library General Public License (file COPYING in
 directory application) for conditions of use and redistribution.
*********************************************************************/

/* For the reliable BCAST version, we need to buffer recently
 * transmitted packets. The required functionality is implemented in
 * the class BCAST_CACHE.
 */

#ifndef _cache_hh_

#define _cache_hh_

#include "bcast.hh"
#include "bcast_const.hh"
#include <sys/time.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <list>

class XrlBcastTarget;
class bcast_cache;

// Each cached entry contains the information about a cached message: PAYLOAD, 
// PAYLOAD LENGTH, and BCAST_HDR

class bcast_cache_entry {
friend class bcast_cache;
public:
  bcast_cache_entry(char *msg, int msg_len, in_addr_t addr, u_int16_t port,
                    struct bcastHeader msg_hdr) 
  { memcpy(payload, msg, msg_len);
    payload_len = msg_len;
    dst_addr = addr;
    dst_port = port;
    memcpy((char *)&bcast_hdr, (char *)&msg_hdr, sizeof(bcastHeader));
  };

private:
  char payload[BCAST_MAX_MSG];      // the intercepted message (payload only
  int payload_len;                  // length of payload
  in_addr_t dst_addr;		    // multicast group that packet is being sent to
  u_int16_t dst_port;               // destination port
  struct bcastHeader bcast_hdr;     // protocol-specific BCAST header
};

// The cache consists primarily of a FIFO list of recently overhead packets.
// It provides a method to add new messages to the cache (ADD_TO_CACHE),
// an INIT_SEND method that will scan the cache for a packets from SRC with
// sequence number SEQ_NO and initiate delayed retransmission, a SEND method
// to actually send the NACK'd packet, and a CANCEL_SEND method to cancel the
// pending retransmission (if we overheard a matching retransmission)

class bcast_cache {
public:
  bcast_cache(XrlBcastTarget *obj){ bcast_obj = obj;};
  void add_to_cache(char *msg, int msg_len, in_addr_t addr, u_int16_t port,
                    struct bcastHeader msg_hdr);
  void init_send(in_addr_t src, u_int32_t seq_no);
  void send();
  void cancel_send(in_addr_t src, u_int32_t seq_no);

private:
  list<bcast_cache_entry> cache;
  bcast_cache_entry *pending;  // which cached message is pending retransmission
  XrlBcastTarget *bcast_obj;  // master routing object, including timers etc.
};


#endif
