/*******************************************************************
 Copyright (C) 2004 Thomas Kunz, CRC Canada, BCAST for IPv4.
 DISTRIBUTED WITH NO WARRANTY, EXPRESS OR IMPLIED.
 See the GNU Library General Public License (file COPYING in
 directory application) for conditions of use and redistribution.
*********************************************************************/

/*
 * Functions to manage neighbor info. We learn about neighbors from
 * periodic HELLO messages, neighbors (and their neighbors) are saved
 * in a list. If the information is not renewed, it will expire soon.
 */

#ifndef _neighbors_hh_
#define _neighbors_hh_

#include <sys/time.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <list>

/*
  Neighbor entry
*/

class Neighbors;
class XrlBcastTarget;

class Neighbor_entry {
public:
        Neighbor_entry(in_addr_t addr, list<in_addr_t> nbs) 
	{ nb_addr = addr; two_hop_nbs = nbs; }

        in_addr_t          nb_addr;
        list<in_addr_t>    two_hop_nbs;
        struct timeval  expire;
};

class Neighbors {
public:
        Neighbors() {}
        void 	nb_insert(in_addr_t addr, list<in_addr_t> nbs, int expiry);
        list<in_addr_t> nb_lookup(in_addr_t addr);
	list<in_addr_t> one_hop_nbs();
	uint	max_degree();
	uint	node_degree();
        void 	nb_purge();

        list<Neighbor_entry> neighbors;
};

void handle_neighbors(XrlBcastTarget *obj, Neighbors *nbs);

#endif
