/*******************************************************************
 Copyright (C) 2004 Thomas Kunz, CRC Canada, BCAST for IPv4.
 DISTRIBUTED WITH NO WARRANTY, EXPRESS OR IMPLIED.
 See the GNU Library General Public License (file COPYING in
 directory application) for conditions of use and redistribution.
*********************************************************************/

/*
 * Functions to manage unique message IDs. These are saved in a list
 * which gets temporarily cleaned out (each entry has a timeout value).
 */

#include "bcast_module.h"
#include "bcast_tgt.hh"

#define CURRENT_TIME(tv)    (void) gettimeofday(&tv, NULL)
#define DEBUG

void
handle_cleanup(XrlBcastTarget *obj, UniqueIDs *ids)
{
  obj->cleanup_t = obj->bcast_events->new_oneoff_after_ms(CLEANUP_PERIOD,
                   callback(handle_cleanup, obj, ids));
  ids->id_purge();
}

/*
   Unique ID Management  Functions
*/

void
UniqueIDs::id_insert(struct bcastHeader bcast_hdr) 
{
  UniqueID_entry *b = 
    new UniqueID_entry(bcast_hdr.sender_addr,bcast_hdr.sequence_no);

  CURRENT_TIME(b->expire);
  b->expire.tv_sec += SAVE_UNIQUE_ID;
  uids.insert(uids.begin(), *b);
}

bool
UniqueIDs::id_lookup(struct bcastHeader bcast_hdr) 
{ // Search the list for a match of source and bid
  list<UniqueID_entry>::iterator iter;
  for (iter  = uids.begin(); iter != uids.end(); iter++) {
    if ((*iter).src == bcast_hdr.sender_addr 
        && (*iter).id == bcast_hdr.sequence_no) {
      return true;
    }
  }
  return false;
}

void
UniqueIDs::id_purge() 
{ struct timeval tv;
  CURRENT_TIME(tv);

#ifdef DEBUG
  fprintf(stderr, "BCAST: purging UNIQUE IDs\n");
#endif

  list<UniqueID_entry>::iterator iter, tmp_iter;
  iter = uids.begin();
  while (iter != uids.end()) {
    tmp_iter = iter;
    tmp_iter++; // save a pointer to the next entry
    if (((*iter).expire.tv_sec < tv.tv_sec) 
	|| ((*iter).expire.tv_sec == tv.tv_sec && 
            (*iter).expire.tv_usec < tv.tv_usec)) {
#ifdef DEBUG
      fprintf(stderr, "BCAST: deleting entry (%3d.%3d.%3d.%3d: %d)",
        NIPQUAD((*iter).src), (*iter).id);
      fprintf(stderr, " with timestamp (%ld, %ld)\n",
        (*iter).expire.tv_sec, (*iter).expire.tv_usec);
#endif
      uids.erase(iter);  // this invalidates the iterator
    }
    iter = tmp_iter;
  }
}

