/*******************************************************************
 Copyright (C) 2004 Thomas Kunz, CRC Canada, BCAST for IPv4.
 DISTRIBUTED WITH NO WARRANTY, EXPRESS OR IMPLIED.
 See the GNU Library General Public License (file COPYING in
 directory application) for conditions of use and redistribution.
*********************************************************************/

/*
 * Functions to manage unique message IDs. These are saved in a list
 * which gets temporarily purged (entries have associated timeout values).
 */

#ifndef _unique_id_hh_
#define _unique_id_hh_

#include "bcast.hh"
#include <sys/time.h>
#include <list>

#define SAVE_UNIQUE_ID 20 	// how many seconds to save each entry 

/*
  Unique ID Cache
*/

class UniqueIDs;
class XrlBcastTarget;

class UniqueID_entry {
	friend class UniqueIDs;
 public:
        UniqueID_entry(in_addr_t i, u_int32_t b) { src = i; id = b;  }
 protected:
        in_addr_t       src;
        u_int32_t       id;
        struct timeval  expire;         // now + SAVE_UNIQUE_ID s
};

class UniqueIDs {
public:
        UniqueIDs() {}
        void 	id_insert(struct bcastHeader bcast_hdr);
        bool 	id_lookup(struct bcastHeader bcast_hdr);
        void 	id_purge();
private:
        list<UniqueID_entry> uids;
};

void handle_cleanup(XrlBcastTarget *obj, UniqueIDs *ids);

#endif
