/*******************************************************************
 Copyright (C) 2004 Thomas Kunz, CRC Canada, BCAST for IPv4.
 DISTRIBUTED WITH NO WARRANTY, EXPRESS OR IMPLIED.
 See the GNU Library General Public License (file COPYING in
 directory application) for conditions of use and redistribution.
*********************************************************************/

// Simple implementation of the BCAST target, for testing purposes

#include "bcast_module.h"

#include "bcast_tgt.hh"
#include "libxorp/status_codes.h"

int XrlBcastTarget::hello_interval = 0;
int XrlBcastTarget::buffer_size = 0;
bool XrlBcastTarget::reliability_level = false;
string XrlBcastTarget::interface = "undef";

XrlCmdError
XrlBcastTarget::common_0_1_get_target_name(string& name)
{
    name = "BCAST Test Target";
    return XrlCmdError::OKAY();
}

XrlCmdError
XrlBcastTarget::common_0_1_get_version(string& version)
{
    version = "1.0";
    return XrlCmdError::OKAY();
}

XrlCmdError
XrlBcastTarget::common_0_1_get_status(uint32_t& status, 
				     string& reason)
{
    status = PROC_READY;
    reason = "Bcast Reason";
    return XrlCmdError::OKAY();
}

XrlCmdError
XrlBcastTarget::common_0_1_shutdown()
{
    exit(0);
}

XrlCmdError XrlBcastTarget::bcast_1_0_set_hello_interval(
	// Input values, 
	const int32_t&	new_int)
{
    hello_interval = new_int;
    return XrlCmdError::OKAY();
}

XrlCmdError XrlBcastTarget::bcast_1_0_get_hello_interval(
	// Output values, 
	int32_t&	old_int)
{
    old_int = hello_interval;
    return XrlCmdError::OKAY();
}

XrlCmdError XrlBcastTarget::bcast_1_0_set_buffer_size(
	// Input values, 
	const int32_t&	new_buf)
{
    buffer_size = new_buf;
    return XrlCmdError::OKAY();
}

XrlCmdError XrlBcastTarget::bcast_1_0_get_buffer_size(
	// Output values, 
	int32_t&	old_buf)
{
    old_buf = buffer_size;
    return XrlCmdError::OKAY();
}

XrlCmdError XrlBcastTarget::bcast_1_0_set_reliability(
	// Input values, 
	const bool&	reliability)
{
    reliability_level = reliability;
    return XrlCmdError::OKAY();
}

XrlCmdError XrlBcastTarget::bcast_1_0_get_reliability(
	// Output values, 
	bool&	reliability)
{
    reliability = reliability_level;
    return XrlCmdError::OKAY();
}

XrlCmdError XrlBcastTarget::bcast_1_0_set_interface(
	// Input values, 
	const string&	intf)
{
    interface = intf;
    return XrlCmdError::OKAY();
}

XrlCmdError XrlBcastTarget::bcast_1_0_get_interface(
	// Output values, 
	string&	intf)
{
    intf = interface;
    return XrlCmdError::OKAY();
}

XrlCmdError XrlBcastTarget::bcast_1_0_activate_bcast()
{
    return XrlCmdError::OKAY();
}

